@@ -40,6 +40,10 @@
 #include "hexdump.c"
 #include "util.h"
 
+int opt_avalon_temp = AVALON_TEMP_TARGET;
+int opt_avalon_overheat = AVALON_TEMP_OVERHEAT;
+bool opt_avalon_auto;
+
 static int option_offset = -1;
 struct device_drv avalon_drv;
 
@@ -49,6 +53,7 @@ static int avalon_init_task(struct avalon_task *at,
 			    uint8_t miner_num, uint8_t nonce_elf,
 			    uint8_t gate_miner, int frequency)
 {
+	uint16_t *lefreq16;
 	uint8_t *buf;
 	static bool first = true;
 
@@ -96,19 +101,8 @@ static int avalon_init_task(struct avalon_task *at,
 	buf[9] = 0x01;
 	buf[10] = 0x00;
 	buf[11] = 0x00;
-	if (frequency == 256) {
-		buf[6] = 0x03;
-		buf[7] = 0x08;
-	} else if (frequency == 270) {
-		buf[6] = 0x73;
-		buf[7] = 0x08;
-	} else if (frequency == 282) {
-		buf[6] = 0xd3;
-		buf[7] = 0x08;
-	} else if (frequency == 300) {
-		buf[6] = 0x63;
-		buf[7] = 0x09;
-	}
+	lefreq16 = (uint16_t *)&buf[6];
+	*lefreq16 = htole16(frequency * 8);
 
 	return 0;
 }
@@ -143,13 +137,11 @@ static int avalon_write(struct cgpu_info *avalon, char *buf, ssize_t len, int ep
 static int avalon_send_task(const struct avalon_task *at, struct cgpu_info *avalon)
 
 {
-	struct timespec p;
 	uint8_t buf[AVALON_WRITE_SIZE + 4 * AVALON_DEFAULT_ASIC_NUM];
-	size_t nr_len;
+	int delay, ret, i, ep = C_AVALON_TASK;
 	struct avalon_info *info;
-	uint64_t delay = 32000000; /* Default 32ms for B19200 */
 	uint32_t nonce_range;
-	int ret, i, ep = C_AVALON_TASK;
+	size_t nr_len;
 
 	if (at->nonce_elf)
 		nr_len = AVALON_WRITE_SIZE + 4 * at->asic_num;
@@ -188,11 +180,9 @@ static int avalon_send_task(const struct avalon_task *at, struct cgpu_info *aval
 	tt |= ((buf[4] & 0x80) ? (1 << 0) : 0);
 	buf[4] = tt;
 #endif
-	if (likely(avalon)) {
-		info = avalon->device_data;
-		delay = nr_len * 10 * 1000000000ULL;
-		delay = delay / info->baud;
-	}
+	info = avalon->device_data;
+	delay = nr_len * 10 * 1000000;
+	delay = delay / info->baud;
 
 	if (at->reset) {
 		ep = C_AVALON_RESET;
@@ -204,10 +194,9 @@ static int avalon_send_task(const struct avalon_task *at, struct cgpu_info *aval
 	}
 	ret = avalon_write(avalon, (char *)buf, nr_len, ep);
 
-	p.tv_sec = 0;
-	p.tv_nsec = (long)delay + 4000000;
-	nanosleep(&p, NULL);
-	applog(LOG_DEBUG, "Avalon: Sent: Buffer delay: %ld", p.tv_nsec);
+	delay += 4000;
+	nusleep(delay);
+	applog(LOG_DEBUG, "Avalon: Sent: Buffer delay: %dus", delay);
 
 	return ret;
 }
@@ -225,8 +214,7 @@ static bool avalon_decode_nonce(struct thr_info *thr, struct cgpu_info *avalon,
 	return submit_nonce(thr, work, nonce);
 }
 
-/* Wait until the ftdi chip returns a CTS saying we can send more data. The
- * status is updated every 40ms. */
+/* Wait until the ftdi chip returns a CTS saying we can send more data. */
 static void wait_avalon_ready(struct cgpu_info *avalon)
 {
 	while (avalon_buffer_full(avalon)) {
@@ -234,23 +222,34 @@ static void wait_avalon_ready(struct cgpu_info *avalon)
 	}
 }
 
+#define AVALON_CTS    (1 << 4)
+
+static inline bool avalon_cts(char c)
+{
+	return (c & AVALON_CTS);
+}
+
 static int avalon_read(struct cgpu_info *avalon, unsigned char *buf,
 		       size_t bufsize, int timeout, int ep)
 {
+	struct avalon_info *info = avalon->device_data;
 	size_t total = 0, readsize = bufsize + 2;
 	char readbuf[AVALON_READBUF_SIZE];
 	int err, amount, ofs = 2, cp;
 
-	/* If the buffer is ready to take more work, yield once in case the
-	 * write thread is waiting to be scheduled. Keep it under the time
-	 * it would take to fill the entire 512 byte buffer. */
-	if (!avalon_buffer_full(avalon))
-		nmsleep(32);
-
 	err = usb_read_once_timeout(avalon, readbuf, readsize, &amount, timeout, ep);
 	applog(LOG_DEBUG, "%s%i: Get avalon read got err %d",
 	       avalon->drv->name, avalon->device_id, err);
 
+	if (amount < 2)
+		goto out;
+
+	/* Use the fact that we're reading the status with the buffer to tell
+	 * the write thread it should send more work without needing to call
+	 * avalon_buffer_full directly. */
+	if (avalon_cts(buf[0]))
+		cgsem_post(&info->write_sem);
+
 	/* The first 2 of every 64 bytes are status on FTDIRL */
 	while (amount > 2) {
 		cp = amount - 2;
@@ -261,6 +260,7 @@ static int avalon_read(struct cgpu_info *avalon, unsigned char *buf,
 		amount -= cp + 2;
 		ofs += 64;
 	}
+out:
 	return total;
 }
 
@@ -441,11 +441,14 @@ static bool get_options(int this_option_offset, int *baud, int *miner_count,
 					case 270:
 					case 282:
 					case 300:
+					case 325:
+					case 350:
+					case 375:
 						*frequency = tmp;
 						break;
 					default:
 						quit(1, "Invalid avalon-options for "
-							"frequency must be 256/270/282/300");
+							"frequency must be 256/270/282/300/325/350/375");
 					}
 				}
 			}
@@ -495,7 +498,7 @@ static void avalon_initialise(struct cgpu_info *avalon)
 
 	// Set latency
 	err = usb_transfer(avalon, FTDI_TYPE_OUT, FTDI_REQUEST_LATENCY,
-				AVALON_READ_TIMEOUT, interface, C_LATENCY);
+			   AVALON_LATENCY, interface, C_LATENCY);
 
 	applog(LOG_DEBUG, "%s%i: latency got err %d",
 		avalon->drv->name, avalon->device_id, err);
@@ -583,6 +586,7 @@ static bool avalon_detect_one(libusb_device *dev, struct usb_find_devices *found
 	/* Even though this is an FTDI type chip, we want to do the parsing
 	 * all ourselves so set it to std usb type */
 	avalon->usbdev->usb_type = USB_TYPE_STD;
+	avalon->usbdev->PrefPacketSize = AVALON_USB_PACKETSIZE;
 
 	/* We have a real Avalon! */
 	avalon_initialise(avalon);
@@ -673,8 +677,8 @@ static void avalon_inc_nvw(struct avalon_info *info, struct thr_info *thr)
 	if (unlikely(info->idle))
 		return;
 
-	applog(LOG_WARNING, "%s%d: No valid work - HW error",
-			thr->cgpu->drv->name, thr->cgpu->device_id);
+	applog(LOG_INFO, "%s%d: No matching work - HW error",
+	       thr->cgpu->drv->name, thr->cgpu->device_id);
 
 	inc_hw_errors(thr);
 	info->no_matching_work++;
@@ -701,6 +705,11 @@ static void avalon_parse_results(struct cgpu_info *avalon, struct avalon_info *i
 				mutex_lock(&info->lock);
 				if (!info->nonces++)
 					gettemp = true;
+				info->auto_nonces++;
+				mutex_unlock(&info->lock);
+			} else if (opt_avalon_auto) {
+				mutex_lock(&info->lock);
+				info->auto_hw++;
 				mutex_unlock(&info->lock);
 			}
 
@@ -744,18 +753,20 @@ static void *avalon_get_results(void *userdata)
 {
 	struct cgpu_info *avalon = (struct cgpu_info *)userdata;
 	struct avalon_info *info = avalon->device_data;
+	int offset = 0, read_delay = 0, ret = 0;
 	const int rsize = AVALON_FTDI_READSIZE;
 	char readbuf[AVALON_READBUF_SIZE];
 	struct thr_info *thr = info->thr;
+	struct timeval tv_start, tv_end;
 	char threadname[24];
-	int offset = 0;
 
 	snprintf(threadname, 24, "ava_recv/%d", avalon->device_id);
 	RenameThread(threadname);
 
 	while (likely(!avalon->shutdown)) {
 		unsigned char buf[rsize];
-		int ret;
+		struct timeval tv_diff;
+		int us_diff;
 
 		if (offset >= (int)AVALON_READ_SIZE)
 			avalon_parse_results(avalon, info, thr, readbuf, &offset);
@@ -767,14 +778,25 @@ static void *avalon_get_results(void *userdata)
 		}
 
 		if (unlikely(info->reset)) {
-			/* Tell the write thread it can start the reset */
-			cgsem_post(&info->write_sem);
-			cgsem_wait(&info->read_sem);
-
+			avalon_running_reset(avalon, info);
 			/* Discard anything in the buffer */
 			offset = 0;
 		}
 
+		/* As the usb read returns after just 1ms, sleep long enough
+		 * to leave the interface idle for writes to occur, but do not
+		 * sleep if we have been receiving data as more may be coming. */
+		if (ret < 1) {
+			cgtime(&tv_end);
+			timersub(&tv_end, &tv_start, &tv_diff);
+			/* Assume it has not been > 1 second so ignore tv_sec */
+			us_diff = tv_diff.tv_usec;
+			read_delay = AVALON_READ_TIMEOUT * 1000 - us_diff;
+			if (likely(read_delay >= 1000))
+				nusleep(read_delay);
+		}
+
+		cgtime(&tv_start);
 		ret = avalon_read(avalon, buf, rsize, AVALON_READ_TIMEOUT,
 				  C_AVALON_READ);
 
@@ -799,6 +821,31 @@ static void avalon_rotate_array(struct cgpu_info *avalon)
 		avalon->work_array = 0;
 }
 
+static void avalon_set_timeout(struct avalon_info *info)
+{
+	info->timeout = AVALON_TIMEOUT_FACTOR / info->frequency;
+}
+
+static void avalon_inc_freq(struct avalon_info *info)
+{
+	info->frequency += 2;
+	if (info->frequency > AVALON_MAX_FREQUENCY)
+		info->frequency = AVALON_MAX_FREQUENCY;
+	avalon_set_timeout(info);
+	applog(LOG_NOTICE, "Avalon increasing frequency to %d, timeout %d",
+	       info->frequency, info->timeout);
+}
+
+static void avalon_dec_freq(struct avalon_info *info)
+{
+	info->frequency -= 1;
+	if (info->frequency < AVALON_MIN_FREQUENCY)
+		info->frequency = AVALON_MIN_FREQUENCY;
+	avalon_set_timeout(info);
+	applog(LOG_NOTICE, "Avalon decreasing frequency to %d, timeout %d",
+	       info->frequency, info->timeout);
+}
+
 static void *avalon_send_tasks(void *userdata)
 {
 	struct cgpu_info *avalon = (struct cgpu_info *)userdata;
@@ -814,14 +861,25 @@ static void *avalon_send_tasks(void *userdata)
 		struct avalon_task at;
 		int idled = 0;
 
-		wait_avalon_ready(avalon);
-
-		if (unlikely(info->reset)) {
-			/* Wait till read thread tells us it's received the
-			 * reset message */
+		while (avalon_buffer_full(avalon))
 			cgsem_wait(&info->write_sem);
-			avalon_running_reset(avalon, info);
-			cgsem_post(&info->read_sem);
+
+		if (opt_avalon_auto && info->auto_queued >= AVALON_AUTO_CYCLE) {
+			mutex_lock(&info->lock);
+			if (info->auto_nonces >= (AVALON_AUTO_CYCLE * 19 / 20) &&
+			    info->auto_nonces <= (AVALON_AUTO_CYCLE * 21 / 20)) {
+				int total = info->auto_nonces + info->auto_hw;
+
+				/* Try to keep hw errors ~1-1.5% */
+				if (info->auto_hw * 100 < total)
+					avalon_inc_freq(info);
+				else if (info->auto_hw * 66 > total)
+					avalon_dec_freq(info);
+			}
+			info->auto_queued =
+			info->auto_nonces =
+			info->auto_hw = 0;
+			mutex_unlock(&info->lock);
 		}
 
 		mutex_lock(&info->qlock);
@@ -835,17 +893,21 @@ static void *avalon_send_tasks(void *userdata)
 				break;
 			}
 
-			if (likely(j < avalon->queued)) {
+			if (likely(j < avalon->queued && !info->overheat)) {
 				info->idle = false;
 				avalon_init_task(&at, 0, 0, info->fan_pwm,
 						info->timeout, info->asic_count,
 						info->miner_count, 1, 0, info->frequency);
 				avalon_create_task(&at, avalon->works[i]);
+				info->auto_queued++;
 			} else {
 				idled++;
 				avalon_init_task(&at, 0, 0, info->fan_pwm,
 						info->timeout, info->asic_count,
 						info->miner_count, 1, 1, info->frequency);
+				/* Reset the auto_queued count if we end up
+				 * idling any miners. */
+				info->auto_queued = 0;
 			}
 
 			ret = avalon_send_task(&at, avalon);
@@ -889,7 +951,6 @@ static bool avalon_prepare(struct thr_info *thr)
 	mutex_init(&info->qlock);
 	if (unlikely(pthread_cond_init(&info->qcond, NULL)))
 		quit(1, "Failed to pthread_cond_init avalon qcond");
-	cgsem_init(&info->read_sem);
 	cgsem_init(&info->write_sem);
 
 	if (pthread_create(&info->read_thr, NULL, avalon_get_results, (void *)avalon))
@@ -916,7 +977,6 @@ static void do_avalon_close(struct thr_info *thr)
 
 	info->no_matching_work = 0;
 
-	cgsem_destroy(&info->read_sem);
 	cgsem_destroy(&info->write_sem);
 }
 
@@ -952,22 +1012,60 @@ static inline void record_temp_fan(struct avalon_info *info, struct avalon_resul
 		info->temp_max = info->temp2;
 }
 
+static void temp_rise(struct avalon_info *info, int temp)
+{
+	if (temp >= opt_avalon_temp + AVALON_TEMP_HYSTERESIS * 3) {
+		info->fan_pwm = AVALON_DEFAULT_FAN_MAX_PWM;
+		return;
+	}
+	if (temp >= opt_avalon_temp + AVALON_TEMP_HYSTERESIS * 2)
+		info->fan_pwm += 10;
+	else if (temp > opt_avalon_temp)
+		info->fan_pwm += 5;
+	else if (temp >= opt_avalon_temp - AVALON_TEMP_HYSTERESIS)
+		info->fan_pwm += 1;
+	else
+		return;
+
+	if (info->fan_pwm > AVALON_DEFAULT_FAN_MAX_PWM)
+		info->fan_pwm = AVALON_DEFAULT_FAN_MAX_PWM;
+}
+
+static void temp_drop(struct avalon_info *info, int temp)
+{
+	if (temp <= opt_avalon_temp - AVALON_TEMP_HYSTERESIS * 3) {
+		info->fan_pwm = AVALON_DEFAULT_FAN_MIN_PWM;
+		return;
+	}
+	if (temp <= opt_avalon_temp - AVALON_TEMP_HYSTERESIS * 2)
+		info->fan_pwm -= 10;
+	else if (temp <= opt_avalon_temp - AVALON_TEMP_HYSTERESIS)
+		info->fan_pwm -= 5;
+	else if (temp < opt_avalon_temp)
+		info->fan_pwm -= 1;
+
+	if (info->fan_pwm < AVALON_DEFAULT_FAN_MIN_PWM)
+		info->fan_pwm = AVALON_DEFAULT_FAN_MIN_PWM;
+}
+
 static inline void adjust_fan(struct avalon_info *info)
 {
 	int temp_new;
 
 	temp_new = info->temp_sum / info->temp_history_count;
 
-	if (temp_new < 35) {
-		info->fan_pwm = AVALON_DEFAULT_FAN_MIN_PWM;
-		info->temp_old = temp_new;
-	} else if (temp_new > 55) {
-		info->fan_pwm = AVALON_DEFAULT_FAN_MAX_PWM;
-		info->temp_old = temp_new;
-	} else if (abs(temp_new - info->temp_old) >= 2) {
-		info->fan_pwm = AVALON_DEFAULT_FAN_MIN_PWM + (temp_new - 35) * 6.4;
-		info->temp_old = temp_new;
+	if (temp_new > info->temp_old)
+		temp_rise(info, temp_new);
+	else if (temp_new < info->temp_old)
+		temp_drop(info, temp_new);
+	else {
+		/* temp_new == info->temp_old */
+		if (temp_new > opt_avalon_temp)
+			temp_rise(info, temp_new);
+		else if (temp_new < opt_avalon_temp - AVALON_TEMP_HYSTERESIS)
+			temp_drop(info, temp_new);
 	}
+	info->temp_old = temp_new;
 }
 
 static void avalon_update_temps(struct cgpu_info *avalon, struct avalon_info *info,
@@ -988,6 +1086,13 @@ static void avalon_update_temps(struct cgpu_info *avalon, struct avalon_info *in
 		info->temp_history_index = 0;
 		info->temp_sum = 0;
 	}
+	if (unlikely(info->temp_old >= opt_avalon_overheat)) {
+		applog(LOG_WARNING, "AVA%d overheat! Idling", avalon->device_id);
+		info->overheat = true;
+	} else if (info->overheat && info->temp_old <= opt_avalon_temp) {
+		applog(LOG_WARNING, "AVA%d cooled, restarting", avalon->device_id);
+		info->overheat = false;
+	}
 }
 
 static void get_avalon_statline_before(char *buf, struct cgpu_info *avalon)
@@ -1045,8 +1150,8 @@ static int64_t avalon_scanhash(struct thr_info *thr)
 	int64_t hash_count, us_timeout;
 	struct timespec abstime;
 
-	/* Full nonce range */
-	us_timeout = 0x100000000ll / info->asic_count / info->frequency;
+	/* Half nonce range */
+	us_timeout = 0x80000000ll / info->asic_count / info->frequency;
 	tdiff.tv_sec = us_timeout / 1000000;
 	tdiff.tv_usec = us_timeout - (tdiff.tv_sec * 1000000);
 	cgtime(&now);
