@@ -26,15 +26,26 @@
 #define AVALON_DEFAULT_FAN_MAX_PWM 0xA0 /* 100% */
 #define AVALON_DEFAULT_FAN_MIN_PWM 0x20 /*  20% */
 
+#define AVALON_TEMP_TARGET 50
+#define AVALON_TEMP_HYSTERESIS 3
+#define AVALON_TEMP_OVERHEAT 60
+
 #define AVALON_DEFAULT_TIMEOUT 0x2D
+#define AVALON_MIN_FREQUENCY 256
+#define AVALON_MAX_FREQUENCY 450
+#define AVALON_TIMEOUT_FACTOR 12000
 #define AVALON_DEFAULT_FREQUENCY 282
 #define AVALON_DEFAULT_MINER_NUM 0x20
 #define AVALON_DEFAULT_ASIC_NUM 0xA
 
+#define AVALON_AUTO_CYCLE 1024
+
 #define AVALON_FTDI_READSIZE 510
+#define AVALON_USB_PACKETSIZE 512
 #define AVALON_READBUF_SIZE 8192
 #define AVALON_RESET_TIMEOUT 100
-#define AVALON_READ_TIMEOUT 10
+#define AVALON_READ_TIMEOUT 18 /* Enough to only half fill the buffer */
+#define AVALON_LATENCY 1
 
 struct avalon_task {
 	uint8_t reset		:1;
@@ -110,12 +121,16 @@ struct avalon_info {
 	pthread_mutex_t lock;
 	pthread_mutex_t qlock;
 	pthread_cond_t qcond;
-	cgsem_t read_sem;
 	cgsem_t write_sem;
 	int nonces;
 
+	int auto_queued;
+	int auto_nonces;
+	int auto_hw;
+
 	bool idle;
 	bool reset;
+	bool overheat;
 };
 
 #define AVALON_WRITE_SIZE (sizeof(struct avalon_task))
@@ -135,6 +150,9 @@ struct avalon_info {
 ASSERT1(sizeof(uint32_t) == 4);
 
 extern struct avalon_info **avalon_info;
+extern int opt_avalon_temp;
+extern int opt_avalon_overheat;
+extern bool opt_avalon_auto;
 
 #endif /* USE_AVALON */
 #endif	/* AVALON_H */
