@@ -30,6 +30,21 @@
 #define BLANK ""
 #define LFSTR "<LF>"
 
+/*
+ * Firmware
+ * DRV_V2 expects (beyond V1) the GetInfo to return the chip count
+ * The queues are 40 instead of 20 and are *usually* consumed and filled
+ * in bursts due to e.g. a 16 chip device doing 16 items at a time and
+ * returning 16 results at a time
+ * If the device has varying chip speeds, it will gradually break up the
+ * burst of results as we progress
+ */
+enum driver_version {
+	BFLSC_DRVUNDEF = 0,
+	BFLSC_DRV1,
+	BFLSC_DRV2
+};
+
 /*
  * With Firmware 1.0.0 and a result queue of 20 the Max is:
  * inprocess = 12
@@ -37,6 +52,14 @@
  * 64+1+24+1+1+(1+8)*8+1 per line = 164 * 20
  * OK = 3
  * Total: 3304
+ *
+ * With Firmware 1.2.* and a result queue of 40 but a limit of 15 replies:
+ * inprocess = 12
+ * max count = 9
+ * 64+1+24+1+1+1+1+(1+8)*8+1 per line = 166 * 15
+ * OK = 3
+ * Total: 2514
+ *
  */
 #define BFLSC_BUFSIZ (0x1000)
 
@@ -49,6 +72,7 @@
 #define BFLSC_DI_XLINKPRESENT "XLINK PRESENT"
 #define BFLSC_DI_DEVICESINCHAIN "DEVICES IN CHAIN"
 #define BFLSC_DI_CHAINPRESENCE "CHAIN PRESENCE MASK"
+#define BFLSC_DI_CHIPS "CHIP PARALLELIZATION"
 
 #define FULLNONCE 0x100000000ULL
 
@@ -73,6 +97,7 @@ struct bflsc_dev {
 	int engines; // each engine represents a 'thread' in a chip
 	char *xlink_mode;
 	char *xlink_present;
+	char *chips;
 
 	// Status
 	bool dead; // TODO: handle seperate x-link devices failing?
@@ -103,6 +128,7 @@ struct bflsc_dev {
 };
 
 struct bflsc_info {
+	enum driver_version driver_version;
 	pthread_rwlock_t stat_lock;
 	struct thr_info results_thr;
 	uint64_t hashes_sent;
@@ -117,6 +143,13 @@ struct bflsc_info {
 	bool flash_led;
 	bool not_first_work; // allow ignoring the first nonce error
 	bool fanauto;
+	int que_size;
+	int que_full_enough;
+	int que_watermark;
+	int que_low;
+	int que_noncecount;
+	int que_fld_min;
+	int que_fld_max;
 };
 
 #define BFLSC_XLINKHDR '@'
@@ -147,9 +180,15 @@ struct QueueJobStructure {
 #define QUE_RES_LINES_MIN 3
 #define QUE_MIDSTATE 0
 #define QUE_BLOCKDATA 1
-#define QUE_NONCECOUNT 2
-#define QUE_FLD_MIN 3
-#define QUE_FLD_MAX 11
+
+#define QUE_NONCECOUNT_V1 2
+#define QUE_FLD_MIN_V1 3
+#define QUE_FLD_MAX_V1 11
+
+#define QUE_CHIP_V2 2
+#define QUE_NONCECOUNT_V2 3
+#define QUE_FLD_MIN_V2 4
+#define QUE_FLD_MAX_V2 12
 
 #define BFLSC_SIGNATURE 0xc1
 #define BFLSC_EOW 0xfe
@@ -171,7 +210,7 @@ struct SaveString {
 	uint8_t payloadData[BFLSC_MAXPAYLOAD];
 };
 
-// Commands
+// Commands (Single Stage)
 #define BFLSC_IDENTIFY "ZGX"
 #define BFLSC_IDENTIFY_LEN (sizeof(BFLSC_IDENTIFY)-1)
 #define BFLSC_DETAILS "ZCX"
@@ -184,15 +223,11 @@ struct SaveString {
 #define BFLSC_VOLTAGE_LEN (sizeof(BFLSC_VOLTAGE)-1)
 #define BFLSC_TEMPERATURE "ZLX"
 #define BFLSC_TEMPERATURE_LEN (sizeof(BFLSC_TEMPERATURE)-1)
-#define BFLSC_QJOB "ZNX"
-#define BFLSC_QJOB_LEN (sizeof(BFLSC_QJOB)-1)
-#define BFLSC_QJOBS "ZWX"
-#define BFLSC_QJOBS_LEN (sizeof(BFLSC_QJOBS)-1)
 #define BFLSC_QRES "ZOX"
 #define BFLSC_QRES_LEN (sizeof(BFLSC_QRES)-1)
 #define BFLSC_QFLUSH "ZQX"
 #define BFLSC_QFLUSH_LEN (sizeof(BFLSC_QFLUSH)-1)
-#define BFLSC_FANAUTO "Z5X"
+#define BFLSC_FANAUTO "Z9X"
 #define BFLSC_FANOUT_LEN (sizeof(BFLSC_FANAUTO)-1)
 #define BFLSC_FAN0 "Z0X"
 #define BFLSC_FAN0_LEN (sizeof(BFLSC_FAN0)-1)
@@ -204,11 +239,17 @@ struct SaveString {
 #define BFLSC_FAN3_LEN (sizeof(BFLSC_FAN3)-1)
 #define BFLSC_FAN4 "Z4X"
 #define BFLSC_FAN4_LEN (sizeof(BFLSC_FAN4)-1)
-#define BFLSC_SAVESTR "ZSX"
-#define BFLSC_SAVESTR_LEN (sizeof(BFLSC_SAVESTR)-1)
 #define BFLSC_LOADSTR "ZUX"
 #define BFLSC_LOADSTR_LEN (sizeof(BFLSC_LOADSTR)-1)
 
+// Commands (Dual Stage)
+#define BFLSC_QJOB "ZNX"
+#define BFLSC_QJOB_LEN (sizeof(BFLSC_QJOB)-1)
+#define BFLSC_QJOBS "ZWX"
+#define BFLSC_QJOBS_LEN (sizeof(BFLSC_QJOBS)-1)
+#define BFLSC_SAVESTR "ZSX"
+#define BFLSC_SAVESTR_LEN (sizeof(BFLSC_SAVESTR)-1)
+
 // Replies
 #define BFLSC_IDENTITY "BitFORCE SC"
 #define BFLSC_BFLSC "SHA256 SC"
@@ -225,6 +266,9 @@ struct SaveString {
 #define BFLSC_ANERR_LEN (sizeof(BFLSC_ANERR)-1)
 #define BFLSC_TIMEOUT BFLSC_ANERR "TIMEOUT"
 #define BFLSC_TIMEOUT_LEN (sizeof(BFLSC_TIMEOUT)-1)
+// x-link timeout has a space (a number follows)
+#define BFLSC_XTIMEOUT BFLSC_ANERR "TIMEOUT "
+#define BFLSC_XTIMEOUT_LEN (sizeof(BFLSC_XTIMEOUT)-1)
 #define BFLSC_INVALID BFLSC_ANERR "INVALID DATA"
 #define BFLSC_INVALID_LEN (sizeof(BFLSC_INVALID)-1)
 #define BFLSC_ERRSIG BFLSC_ANERR "SIGNATURE"
@@ -245,11 +289,9 @@ struct SaveString {
 #define FullNonceRangeJob QueueJobStructure
 #define BFLSC_JOBSIZ BFLSC_QJOBSIZ
 
-// Non queued commands
+// Non queued commands (not used)
 #define BFLSC_SENDWORK "ZDX"
 #define BFLSC_SENDWORK_LEN (sizeof(BFLSC_SENDWORK)-1)
-
-// Non queued commands (not used)
 #define BFLSC_WORKSTATUS "ZFX"
 #define BFLSC_WORKSTATUS_LEN (sizeof(BFLSC_WORKSTATUS)-1)
 #define BFLSC_SENDRANGE "ZPX"
@@ -294,14 +336,24 @@ struct SaveString {
 #define BFLSC_MAX_SLEEP 2000
 
 #define BAJ_LATENCY LATENCY_STD
-#define BAL_LATENCY LATENCY_STD
-#define BAS_LATENCY LATENCY_STD
+#define BAL_LATENCY 12
+#define BAS_LATENCY 12
+// For now a BAM doesn't really exist - it's currently 8 independent BASs
 #define BAM_LATENCY 2
 
 #define BFLSC_TEMP_SLEEPMS 5
-#define BFLSC_QUE_SIZE 20
-#define BFLSC_QUE_FULL_ENOUGH 13
-#define BFLSC_QUE_WATERMARK 6
+
+#define BFLSC_QUE_SIZE_V1 20
+#define BFLSC_QUE_FULL_ENOUGH_V1 13
+#define BFLSC_QUE_WATERMARK_V1 6
+#define BFLSC_QUE_LOW_V1 2
+
+// TODO: use 5 batch jobs
+// TODO: base these numbers on the chip count?
+#define BFLSC_QUE_SIZE_V2 40
+#define BFLSC_QUE_FULL_ENOUGH_V2 36
+#define BFLSC_QUE_WATERMARK_V2 32
+#define BFLSC_QUE_LOW_V2 8
 
 // Must drop this far below cutoff before resuming work
 #define BFLSC_TEMP_RECOVER 5
@@ -318,6 +370,29 @@ static const char *blank = "";
 
 struct device_drv bflsc_drv;
 
+static enum driver_version drv_ver(struct cgpu_info *bflsc, const char *ver)
+{
+	char *tmp;
+
+	if (strcmp(ver, "1.0.0") == 0)
+		return BFLSC_DRV1;
+
+	if (strncmp(ver, "1.0", 3) == 0 || strncmp(ver, "1.1", 3) == 0) {
+		applog(LOG_WARNING, "%s detect (%s) Warning assuming firmware '%s' is Ver1",
+			bflsc->drv->dname, bflsc->device_path, ver);
+		return BFLSC_DRV1;
+	}
+
+	if (strncmp(ver, "1.2", 3) == 0)
+		return BFLSC_DRV2;
+
+	tmp = str_text((char *)ver);
+	applog(LOG_WARNING, "%s detect (%s) Warning unknown firmware '%s' using Ver2",
+		bflsc->drv->dname, bflsc->device_path, tmp);
+	free(tmp);
+	return BFLSC_DRV2;
+}
+
 static void xlinkstr(char *xlink, int dev, struct bflsc_info *sc_info)
 {
 	if (dev > 0)
@@ -469,6 +544,174 @@ static void freebreakdown(int *count, char **firstname, char ***fields)
 	*fields = NULL;
 }
 
+static bool isokerr(int err, char *buf, int amount)
+{
+	if (err < 0 || amount < (int)BFLSC_OK_LEN)
+		return false;
+	else {
+		if (strncmp(buf, BFLSC_ANERR, BFLSC_ANERR_LEN) == 0)
+			return false;
+		else
+			return true;
+	}
+}
+
+// send+receive dual stage - always single line replies
+static int send_recv_ds(struct cgpu_info *bflsc, int dev, int *stage, bool *sent, int *amount, char *send1, int send1_len, enum usb_cmds send1_cmd,  enum usb_cmds recv1_cmd, char *send2, int send2_len, enum usb_cmds send2_cmd, enum usb_cmds recv2_cmd, char *recv, int recv_siz)
+{
+	struct DataForwardToChain data;
+	int len, err, tried;
+
+	if (dev == 0) {
+		usb_buffer_clear(bflsc);
+
+		*stage = 1;
+		*sent = false;
+		err = usb_write(bflsc, send1, send1_len, amount, send1_cmd);
+		if (err < 0 || *amount < send1_len)
+			return err;
+
+		*sent = true;
+		err = usb_read_nl(bflsc, recv, recv_siz, amount, recv1_cmd);
+		if (!isokerr(err, recv, *amount))
+			return err;
+
+		usb_buffer_clear(bflsc);
+
+		*stage = 2;
+		*sent = false;
+		err = usb_write(bflsc, send2, send2_len, amount, send2_cmd);
+		if (err < 0 || *amount < send2_len)
+			return err;
+
+		*sent = true;
+		err = usb_read_nl(bflsc, recv, recv_siz, amount, recv2_cmd);
+
+		return err;
+	}
+
+	data.header = BFLSC_XLINKHDR;
+	data.deviceAddress = (uint8_t)dev;
+	tried = 0;
+	while (tried++ < 3) {
+		data.payloadSize = send1_len;
+		memcpy(data.payloadData, send1, send1_len);
+		len = DATAFORWARDSIZE(data);
+
+		usb_buffer_clear(bflsc);
+
+		*stage = 1;
+		*sent = false;
+		err = usb_write(bflsc, (char *)&data, len, amount, send1_cmd);
+		if (err < 0 || *amount < send1_len)
+			return err;
+
+		*sent = true;
+		err = usb_read_nl(bflsc, recv, recv_siz, amount, recv1_cmd);
+
+		if (err != LIBUSB_SUCCESS)
+			return err;
+
+		// x-link timeout? - try again?
+		if (strncasecmp(recv, BFLSC_XTIMEOUT, BFLSC_XTIMEOUT_LEN) == 0)
+			continue;
+
+		if (!isokerr(err, recv, *amount))
+			return err;
+
+		data.payloadSize = send2_len;
+		memcpy(data.payloadData, send2, send2_len);
+		len = DATAFORWARDSIZE(data);
+
+		usb_buffer_clear(bflsc);
+
+		*stage = 2;
+		*sent = false;
+		err = usb_write(bflsc, (char *)&data, len, amount, send2_cmd);
+		if (err < 0 || *amount < send2_len)
+			return err;
+
+		*sent = true;
+		err = usb_read_nl(bflsc, recv, recv_siz, amount, recv2_cmd);
+
+		if (err != LIBUSB_SUCCESS)
+			return err;
+
+		// x-link timeout? - try again?
+		if (strncasecmp(recv, BFLSC_XTIMEOUT, BFLSC_XTIMEOUT_LEN) == 0)
+			continue;
+
+		// SUCCESS - return it
+		break;
+	}
+	return err;
+}
+
+#define READ_OK true
+#define READ_NL false
+
+// send+receive single stage
+static int send_recv_ss(struct cgpu_info *bflsc, int dev, bool *sent, int *amount, char *send, int send_len, enum usb_cmds send_cmd, char *recv, int recv_siz, enum usb_cmds recv_cmd, bool read_ok)
+{
+	struct DataForwardToChain data;
+	int len, err, tried;
+
+	if (dev == 0) {
+		usb_buffer_clear(bflsc);
+
+		*sent = false;
+		err = usb_write(bflsc, send, send_len, amount, send_cmd);
+		if (err < 0 || *amount < send_len) {
+			// N.B. thus !(*sent) directly implies err < 0 or *amount < send_len
+			return err;
+		}
+
+		*sent = true;
+		if (read_ok == READ_OK)
+			err = usb_read_ok(bflsc, recv, recv_siz, amount, recv_cmd);
+		else
+			err = usb_read_nl(bflsc, recv, recv_siz, amount, recv_cmd);
+
+		return err;
+	}
+
+	data.header = BFLSC_XLINKHDR;
+	data.deviceAddress = (uint8_t)dev;
+	data.payloadSize = send_len;
+	memcpy(data.payloadData, send, send_len);
+	len = DATAFORWARDSIZE(data);
+
+	tried = 0;
+	while (tried++ < 3) {
+		usb_buffer_clear(bflsc);
+
+		*sent = false;
+		err = usb_write(bflsc, (char *)&data, len, amount, recv_cmd);
+		if (err < 0 || *amount < send_len)
+			return err;
+
+		*sent = true;
+		if (read_ok == READ_OK)
+			err = usb_read_ok(bflsc, recv, recv_siz, amount, recv_cmd);
+		else
+			err = usb_read_nl(bflsc, recv, recv_siz, amount, recv_cmd);
+
+		if (err != LIBUSB_SUCCESS && err != LIBUSB_ERROR_TIMEOUT)
+			return err;
+
+		// read_ok can err timeout if it's looking for OK<LF>
+		// TODO: add a usb_read() option to spot the ERR: and convert end=OK<LF> to just <LF>
+		// x-link timeout? - try again?
+		if ((err == LIBUSB_SUCCESS || (read_ok == READ_OK && err == LIBUSB_ERROR_TIMEOUT)) &&
+			strncasecmp(recv, BFLSC_XTIMEOUT, BFLSC_XTIMEOUT_LEN) == 0)
+				continue;
+
+		// SUCCESS or TIMEOUT - return it
+		break;
+	}
+	return err;
+}
+
 static int write_to_dev(struct cgpu_info *bflsc, int dev, char *buf, int buflen, int *amount, enum usb_cmds cmd)
 {
 	struct DataForwardToChain data;
@@ -489,53 +732,29 @@ static int write_to_dev(struct cgpu_info *bflsc, int dev, char *buf, int buflen,
 	memcpy(data.payloadData, buf, buflen);
 	len = DATAFORWARDSIZE(data);
 
-	// TODO: handle xlink timeout message - here or at call?
 	return usb_write(bflsc, (char *)&data, len, amount, cmd);
 }
 
-static bool getok(struct cgpu_info *bflsc, enum usb_cmds cmd, int *err, int *amount)
-{
-	char buf[BFLSC_BUFSIZ+1];
-
-	*err = usb_read_nl(bflsc, buf, sizeof(buf)-1, amount, cmd);
-	if (*err < 0 || *amount < (int)BFLSC_OK_LEN)
-		return false;
-	else
-		return true;
-}
-
-static bool getokerr(struct cgpu_info *bflsc, enum usb_cmds cmd, int *err, int *amount, char *buf, size_t bufsiz)
-{
-	*err = usb_read_nl(bflsc, buf, bufsiz-1, amount, cmd);
-	if (*err < 0 || *amount < (int)BFLSC_OK_LEN)
-		return false;
-	else {
-		if (*amount > (int)BFLSC_ANERR_LEN && strncmp(buf, BFLSC_ANERR, BFLSC_ANERR_LEN) == 0)
-			return false;
-		else
-			return true;
-	}
-}
-
 static void bflsc_send_flush_work(struct cgpu_info *bflsc, int dev)
 {
+	char buf[BFLSC_BUFSIZ+1];
 	int err, amount;
+	bool sent;
 
 	// Device is gone
 	if (bflsc->usbinfo.nodev)
 		return;
 
 	mutex_lock(&bflsc->device_mutex);
+	err = send_recv_ss(bflsc, dev, &sent, &amount,
+				BFLSC_QFLUSH, BFLSC_QFLUSH_LEN, C_QUEFLUSH,
+				buf, sizeof(buf)-1, C_QUEFLUSHREPLY, READ_NL);
+	mutex_unlock(&bflsc->device_mutex);
 
-	err = write_to_dev(bflsc, dev, BFLSC_QFLUSH, BFLSC_QFLUSH_LEN, &amount, C_QUEFLUSH);
-	if (err < 0 || amount != BFLSC_QFLUSH_LEN) {
-		mutex_unlock(&bflsc->device_mutex);
+	if (!sent)
 		bflsc_applog(bflsc, dev, C_QUEFLUSH, amount, err);
-	} else {
+	else {
 		// TODO: do we care if we don't get 'OK'? (always will in normal processing)
-		err = getok(bflsc, C_QUEFLUSHREPLY, &err, &amount);
-		mutex_unlock(&bflsc->device_mutex);
-		// TODO: report an error if not 'OK' ?
 	}
 }
 
@@ -546,10 +765,12 @@ static bool bflsc_qres(struct cgpu_info *bflsc, char *buf, size_t bufsiz, int de
 	bool readok = false;
 
 	mutex_lock(&(bflsc->device_mutex));
+	*err = send_recv_ss(bflsc, dev, &readok, amount,
+				BFLSC_QRES, BFLSC_QRES_LEN, C_REQUESTRESULTS,
+				buf, bufsiz-1, C_GETRESULTS, READ_OK);
+	mutex_unlock(&(bflsc->device_mutex));
 
-	*err = write_to_dev(bflsc, dev, BFLSC_QRES, BFLSC_QRES_LEN, amount, C_REQUESTRESULTS);
-	if (*err < 0 || *amount != BFLSC_QRES_LEN) {
-		mutex_unlock(&(bflsc->device_mutex));
+	if (!readok) {
 		if (!ignore)
 			bflsc_applog(bflsc, dev, C_REQUESTRESULTS, *amount, *err);
 
@@ -559,10 +780,6 @@ static bool bflsc_qres(struct cgpu_info *bflsc, char *buf, size_t bufsiz, int de
 		// has failed after some limit of this?
 		// of course all other I/O must also be failing ...
 	} else {
-		readok = true;
-		*err = usb_read_ok(bflsc, buf, bufsiz-1, amount, C_GETRESULTS);
-		mutex_unlock(&(bflsc->device_mutex));
-
 		if (*err < 0 || *amount < 1) {
 			if (!ignore)
 				bflsc_applog(bflsc, dev, C_GETRESULTS, *amount, *err);
@@ -658,7 +875,7 @@ static void __bflsc_initialise(struct cgpu_info *bflsc)
 		bflsc->drv->name, bflsc->device_id, err);
 
 	if (!bflsc->cutofftemp)
-		bflsc->cutofftemp = 80;
+		bflsc->cutofftemp = 90;
 }
 
 static void bflsc_initialise(struct cgpu_info *bflsc)
@@ -685,8 +902,8 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 	char buf[BFLSC_BUFSIZ+1];
 	int err, amount;
 	char **items, *firstname, **fields, *lf;
+	bool res, ok = false;
 	int i, lines, count;
-	bool res, ok;
 	char *tmp;
 
 	/*
@@ -702,12 +919,15 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 	 * OK<LF>
 	 */
 
-	// TODO: if dev is ever > 0 must handle xlink timeout message
+	/*
+	 * Don't use send_recv_ss() since we have a different receive timeout
+	 * Also getinfo() is called multiple times if it fails anyway
+	 */
 	err = write_to_dev(bflsc, dev, BFLSC_DETAILS, BFLSC_DETAILS_LEN, &amount, C_REQUESTDETAILS);
 	if (err < 0 || amount != BFLSC_DETAILS_LEN) {
 		applog(LOG_ERR, "%s detect (%s) send details request failed (%d:%d)",
 			bflsc->drv->dname, bflsc->device_path, amount, err);
-		return false;
+		return ok;
 	}
 
 	err = usb_read_ok_timeout(bflsc, buf, sizeof(buf)-1, &amount,
@@ -720,14 +940,14 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 			applog(LOG_ERR, "%s detect (%s) get details returned nothing (%d:%d)",
 					bflsc->drv->dname, bflsc->device_path, amount, err);
 		}
-		return false;
+		return ok;
 	}
 
 	memset(&sc_dev, 0, sizeof(struct bflsc_dev));
 	sc_info->sc_count = 1;
 	res = tolines(bflsc, dev, &(buf[0]), &lines, &items, C_GETDETAILS);
 	if (!res)
-		return false;
+		return ok;
 
 	tmp = str_text(buf);
 	strcpy(sc_dev.getinfo, tmp);
@@ -747,12 +967,7 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 		}
 		if (strcmp(firstname, BFLSC_DI_FIRMWARE) == 0) {
 			sc_dev.firmware = strdup(fields[0]);
-			if (strcmp(sc_dev.firmware, "1.0.0")) {
-				tmp = str_text(items[i]);
-				applog(LOG_WARNING, "%s detect (%s) Warning unknown firmware '%s'",
-					bflsc->drv->dname, bflsc->device_path, tmp);
-				free(tmp);
-			}
+			sc_info->driver_version = drv_ver(bflsc, sc_dev.firmware);
 		}
 		else if (strcmp(firstname, BFLSC_DI_ENGINES) == 0) {
 			sc_dev.engines = atoi(fields[0]);
@@ -769,7 +984,7 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 		else if (strcmp(firstname, BFLSC_DI_XLINKPRESENT) == 0)
 			sc_dev.xlink_present = strdup(fields[0]);
 		else if (strcmp(firstname, BFLSC_DI_DEVICESINCHAIN) == 0) {
-			sc_info->sc_count = atoi(fields[0]) + 1;
+			sc_info->sc_count = atoi(fields[0]);
 			if (sc_info->sc_count < 1 || sc_info->sc_count > 30) {
 				tmp = str_text(items[i]);
 				applog(LOG_WARNING, "%s detect (%s) invalid s-link count: '%s'",
@@ -777,10 +992,18 @@ static bool getinfo(struct cgpu_info *bflsc, int dev)
 				free(tmp);
 				goto mata;
 			}
+		else if (strcmp(firstname, BFLSC_DI_CHIPS) == 0)
+			sc_dev.chips = strdup(fields[0]);
 		}
 		freebreakdown(&count, &firstname, &fields);
 	}
 
+	if (sc_info->driver_version == BFLSC_DRVUNDEF) {
+		applog(LOG_WARNING, "%s detect (%s) missing %s",
+			bflsc->drv->dname, bflsc->device_path, BFLSC_DI_FIRMWARE);
+		goto ne;
+	}
+
 	sc_info->sc_devs = calloc(sc_info->sc_count, sizeof(struct bflsc_dev));
 	if (unlikely(!sc_info->sc_devs))
 		quit(1, "Failed to calloc in getinfo");
@@ -805,7 +1028,7 @@ static bool bflsc_detect_one(struct libusb_device *dev, struct usb_find_devices
 	int i, err, amount;
 	struct timeval init_start, init_now;
 	int init_sleep, init_count;
-	bool ident_first;
+	bool ident_first, sent;
 	char *newname;
 	uint16_t latency;
 
@@ -828,14 +1051,17 @@ retry:
 	cgtime(&init_start);
 reinit:
 	__bflsc_initialise(bflsc);
-	err = write_to_dev(bflsc, 0, BFLSC_IDENTIFY, BFLSC_IDENTIFY_LEN, &amount, C_REQUESTIDENTIFY);
-	if (err < 0 || amount != BFLSC_IDENTIFY_LEN) {
+
+	err = send_recv_ss(bflsc, 0, &sent, &amount,
+				BFLSC_IDENTIFY, BFLSC_IDENTIFY_LEN, C_REQUESTIDENTIFY,
+				buf, sizeof(buf)-1, C_GETIDENTIFY, READ_NL);
+
+	if (!sent) {
 		applog(LOG_ERR, "%s detect (%s) send identify request failed (%d:%d)",
 			bflsc->drv->dname, bflsc->device_path, amount, err);
 		goto unshin;
 	}
 
-	err = usb_read_nl(bflsc, buf, sizeof(buf)-1, &amount, C_GETIDENTIFY);
 	if (err < 0 || amount < 1) {
 		init_count++;
 		cgtime(&init_now);
@@ -884,8 +1110,42 @@ reinit:
 		goto unshin;
 	}
 
-	if (!getinfo(bflsc, 0))
-		goto unshin;
+	int tries = 0;
+	while (7734) {
+		if (getinfo(bflsc, 0))
+			break;
+
+		// N.B. we will get displayed errors each time it fails
+		if (++tries > 2)
+			goto unshin;
+
+		nmsleep(40);
+	}
+
+	switch (sc_info->driver_version) {
+		case BFLSC_DRV1:
+			sc_info->que_size = BFLSC_QUE_SIZE_V1;
+			sc_info->que_full_enough = BFLSC_QUE_FULL_ENOUGH_V1;
+			sc_info->que_watermark = BFLSC_QUE_WATERMARK_V1;
+			sc_info->que_low = BFLSC_QUE_LOW_V1;
+			sc_info->que_noncecount = QUE_NONCECOUNT_V1;
+			sc_info->que_fld_min = QUE_FLD_MIN_V1;
+			sc_info->que_fld_max = QUE_FLD_MAX_V1;
+			break;
+		case BFLSC_DRV2:
+		case BFLSC_DRVUNDEF:
+		default:
+			sc_info->driver_version = BFLSC_DRV2;
+
+			sc_info->que_size = BFLSC_QUE_SIZE_V2;
+			sc_info->que_full_enough = BFLSC_QUE_FULL_ENOUGH_V2;
+			sc_info->que_watermark = BFLSC_QUE_WATERMARK_V2;
+			sc_info->que_low = BFLSC_QUE_LOW_V2;
+			sc_info->que_noncecount = QUE_NONCECOUNT_V2;
+			sc_info->que_fld_min = QUE_FLD_MIN_V2;
+			sc_info->que_fld_max = QUE_FLD_MAX_V2;
+			break;
+	}
 
 	sc_info->scan_sleep_time = BAS_SCAN_TIME;
 	sc_info->results_sleep_time = BAS_RES_TIME;
@@ -1037,7 +1297,9 @@ static void bflsc_flush_work(struct cgpu_info *bflsc)
 static void bflsc_flash_led(struct cgpu_info *bflsc, int dev)
 {
 	struct bflsc_info *sc_info = (struct bflsc_info *)(bflsc->device_data);
+	char buf[BFLSC_BUFSIZ+1];
 	int err, amount;
+	bool sent;
 
 	// Device is gone
 	if (bflsc->usbinfo.nodev)
@@ -1048,14 +1310,15 @@ static void bflsc_flash_led(struct cgpu_info *bflsc, int dev)
 	if (mutex_trylock(&bflsc->device_mutex))
 		return;
 
-	err = write_to_dev(bflsc, dev, BFLSC_FLASH, BFLSC_FLASH_LEN, &amount, C_REQUESTFLASH);
-	if (err < 0 || amount != BFLSC_FLASH_LEN) {
-		mutex_unlock(&(bflsc->device_mutex));
-		bflsc_applog(bflsc, dev, C_REQUESTFLASH, amount, err);
-	} else {
-		getok(bflsc, C_FLASHREPLY, &err, &amount);
+	err = send_recv_ss(bflsc, dev, &sent, &amount,
+				BFLSC_FLASH, BFLSC_FLASH_LEN, C_REQUESTFLASH,
+				buf, sizeof(buf)-1, C_FLASHREPLY, READ_NL);
+	mutex_unlock(&(bflsc->device_mutex));
 
-		mutex_unlock(&(bflsc->device_mutex));
+	if (!sent)
+		bflsc_applog(bflsc, dev, C_REQUESTFLASH, amount, err);
+	else {
+		// Don't care
 	}
 
 	// Once we've tried - don't do it until told to again
@@ -1076,7 +1339,7 @@ static bool bflsc_get_temp(struct cgpu_info *bflsc, int dev)
 	char *firstname, **fields, *lf;
 	char xlink[17];
 	int count;
-	bool res;
+	bool res, sent;
 	float temp, temp1, temp2;
 	float vcc1, vcc2, vmain;
 
@@ -1096,58 +1359,61 @@ static bool bflsc_get_temp(struct cgpu_info *bflsc, int dev)
 		return true;
 	}
 
+	xlinkstr(&(xlink[0]), dev, sc_info);
+
 	/* It is not very critical getting temp so don't get stuck if we
 	 * can't grab the mutex here */
 	if (mutex_trylock(&bflsc->device_mutex))
 		return false;
 
-	xlinkstr(&(xlink[0]), dev, sc_info);
+	err = send_recv_ss(bflsc, dev, &sent, &amount,
+				BFLSC_TEMPERATURE, BFLSC_TEMPERATURE_LEN, C_REQUESTTEMPERATURE,
+				temp_buf, sizeof(temp_buf)-1, C_GETTEMPERATURE, READ_NL);
+	mutex_unlock(&(bflsc->device_mutex));
 
-	err = write_to_dev(bflsc, dev, BFLSC_TEMPERATURE, BFLSC_TEMPERATURE_LEN, &amount, C_REQUESTTEMPERATURE);
-	if (err < 0 || amount != BFLSC_TEMPERATURE_LEN) {
-		mutex_unlock(&(bflsc->device_mutex));
+	if (!sent) {
 		applog(LOG_ERR, "%s%i: Error: Request%s temp invalid/timed out (%d:%d)",
 				bflsc->drv->name, bflsc->device_id, xlink, amount, err);
 		return false;
-	}
-
-	err = usb_read_nl(bflsc, temp_buf, sizeof(temp_buf)-1, &amount, C_GETTEMPERATURE);
-	if (err < 0 || amount < 1) {
-		mutex_unlock(&(bflsc->device_mutex));
-		if (err < 0) {
-			applog(LOG_ERR, "%s%i: Error: Get%s temp return invalid/timed out (%d:%d)",
-					bflsc->drv->name, bflsc->device_id, xlink, amount, err);
-		} else {
-			applog(LOG_ERR, "%s%i: Error: Get%s temp returned nothing (%d:%d)",
-					bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+	} else {
+		if (err < 0 || amount < 1) {
+			if (err < 0) {
+				applog(LOG_ERR, "%s%i: Error: Get%s temp return invalid/timed out (%d:%d)",
+						bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+			} else {
+				applog(LOG_ERR, "%s%i: Error: Get%s temp returned nothing (%d:%d)",
+						bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+			}
+			return false;
 		}
-		return false;
 	}
 
-	// N.B. we only get the voltages if the temp succeeds - temp is the important one
-	err = write_to_dev(bflsc, dev, BFLSC_VOLTAGE, BFLSC_VOLTAGE_LEN, &amount, C_REQUESTVOLTS);
-	if (err < 0 || amount != BFLSC_VOLTAGE_LEN) {
-		mutex_unlock(&(bflsc->device_mutex));
+	// Ignore it if we can't get the V
+	if (mutex_trylock(&bflsc->device_mutex))
+		return false;
+
+	err = send_recv_ss(bflsc, dev, &sent, &amount,
+				BFLSC_VOLTAGE, BFLSC_VOLTAGE_LEN, C_REQUESTVOLTS,
+				volt_buf, sizeof(volt_buf)-1, C_GETTEMPERATURE, READ_NL);
+	mutex_unlock(&(bflsc->device_mutex));
+
+	if (!sent) {
 		applog(LOG_ERR, "%s%i: Error: Request%s volts invalid/timed out (%d:%d)",
 				bflsc->drv->name, bflsc->device_id, xlink, amount, err);
 		return false;
-	}
-
-	err = usb_read_nl(bflsc, volt_buf, sizeof(volt_buf)-1, &amount, C_GETTEMPERATURE);
-	if (err < 0 || amount < 1) {
-		mutex_unlock(&(bflsc->device_mutex));
-		if (err < 0) {
-			applog(LOG_ERR, "%s%i: Error: Get%s temp return invalid/timed out (%d:%d)",
-					bflsc->drv->name, bflsc->device_id, xlink, amount, err);
-		} else {
-			applog(LOG_ERR, "%s%i: Error: Get%s temp returned nothing (%d:%d)",
-					bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+	} else {
+		if (err < 0 || amount < 1) {
+			if (err < 0) {
+				applog(LOG_ERR, "%s%i: Error: Get%s volt return invalid/timed out (%d:%d)",
+						bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+			} else {
+				applog(LOG_ERR, "%s%i: Error: Get%s volt returned nothing (%d:%d)",
+						bflsc->drv->name, bflsc->device_id, xlink, amount, err);
+			}
+			return false;
 		}
-		return false;
 	}
 
-	mutex_unlock(&(bflsc->device_mutex));
-	
 	res = breakdown(ALLCOLON, temp_buf, &count, &firstname, &fields, &lf);
 	if (lf)
 		*lf = '\0';
@@ -1164,6 +1430,8 @@ static bool bflsc_get_temp(struct cgpu_info *bflsc, int dev)
 	temp = temp1 = (float)atoi(fields[0]);
 	temp2 = (float)atoi(fields[1]);
 
+	freebreakdown(&count, &firstname, &fields);
+
 	res = breakdown(NOCOLON, volt_buf, &count, &firstname, &fields, &lf);
 	if (lf)
 		*lf = '\0';
@@ -1181,6 +1449,9 @@ static bool bflsc_get_temp(struct cgpu_info *bflsc, int dev)
 	vcc1 = (float)atoi(fields[0]) / 1000.0;
 	vcc2 = (float)atoi(fields[1]) / 1000.0;
 	vmain = (float)atoi(fields[2]) / 1000.0;
+
+	freebreakdown(&count, &firstname, &fields);
+
 	if (vcc1 > 0 || vcc2 > 0 || vmain > 0) {
 		wr_lock(&(sc_info->stat_lock));
 		if (vcc1 > 0) {
@@ -1266,7 +1537,6 @@ static bool bflsc_get_temp(struct cgpu_info *bflsc, int dev)
 			sc_dev->overheat = false;
 	}
 
-	freebreakdown(&count, &firstname, &fields);
 	return true;
 }
 
@@ -1280,27 +1550,27 @@ static void process_nonces(struct cgpu_info *bflsc, int dev, char *xlink, char *
 	bool res;
 	char *tmp;
 
-	if (count < QUE_FLD_MIN) {
+	if (count < sc_info->que_fld_min) {
 		tmp = str_text(data);
-		applog(LOG_ERR, "%s%i:%s work returned too small (%d,%s)",
+		applog(LOG_INFO, "%s%i:%s work returned too small (%d,%s)",
 				bflsc->drv->name, bflsc->device_id, xlink, count, tmp);
 		free(tmp);
 		inc_hw_errors(bflsc->thr[0]);
 		return;
 	}
 
-	if (count > QUE_FLD_MAX) {
-		applog(LOG_ERR, "%s%i:%s work returned too large (%d) processing %d anyway",
-				bflsc->drv->name, bflsc->device_id, xlink, count, QUE_FLD_MAX);
-		count = QUE_FLD_MAX;
+	if (count > sc_info->que_fld_max) {
+		applog(LOG_INFO, "%s%i:%s work returned too large (%d) processing %d anyway",
+		       bflsc->drv->name, bflsc->device_id, xlink, count, sc_info->que_fld_max);
+		count = sc_info->que_fld_max;
 		inc_hw_errors(bflsc->thr[0]);
 	}
 
-	num = atoi(fields[QUE_NONCECOUNT]);
-	if (num != count - QUE_FLD_MIN) {
+	num = atoi(fields[sc_info->que_noncecount]);
+	if (num != count - sc_info->que_fld_min) {
 		tmp = str_text(data);
-		applog(LOG_ERR, "%s%i:%s incorrect data count (%d) will use %d instead from (%s)",
-				bflsc->drv->name, bflsc->device_id, xlink, num, count - QUE_FLD_MAX, tmp);
+		applog(LOG_INFO, "%s%i:%s incorrect data count (%d) will use %d instead from (%s)",
+		       bflsc->drv->name, bflsc->device_id, xlink, num, count - sc_info->que_fld_max, tmp);
 		free(tmp);
 		inc_hw_errors(bflsc->thr[0]);
 	}
@@ -1309,7 +1579,7 @@ static void process_nonces(struct cgpu_info *bflsc, int dev, char *xlink, char *
 	memset(blockdata, 0, MERKLE_BYTES);
 	if (!hex2bin((unsigned char *)midstate, fields[QUE_MIDSTATE], MIDSTATE_BYTES) ||
 	    !hex2bin((unsigned char *)blockdata, fields[QUE_BLOCKDATA], MERKLE_BYTES)) {
-		applog(LOG_ERR, "%s%i:%s Failed to convert binary data to hex result - ignored",
+		applog(LOG_INFO, "%s%i:%s Failed to convert binary data to hex result - ignored",
 		       bflsc->drv->name, bflsc->device_id, xlink);
 		inc_hw_errors(bflsc->thr[0]);
 		return;
@@ -1319,19 +1589,19 @@ static void process_nonces(struct cgpu_info *bflsc, int dev, char *xlink, char *
 						blockdata, MERKLE_OFFSET, MERKLE_BYTES);
 	if (!work) {
 		if (sc_info->not_first_work) {
-			applog(LOG_ERR, "%s%i:%s failed to find nonce work - can't be processed - ignored",
-					bflsc->drv->name, bflsc->device_id, xlink);
-					inc_hw_errors(bflsc->thr[0]);
+			applog(LOG_INFO, "%s%i:%s failed to find nonce work - can't be processed - ignored",
+			       bflsc->drv->name, bflsc->device_id, xlink);
+			inc_hw_errors(bflsc->thr[0]);
 		}
 		return;
 	}
 
 	res = false;
-	for (i = QUE_FLD_MIN; i < count; i++) {
+	for (i = sc_info->que_fld_min; i < count; i++) {
 		if (strlen(fields[i]) != 8) {
 			tmp = str_text(data);
-			applog(LOG_ERR, "%s%i:%s invalid nonce (%s) will try to process anyway",
-					bflsc->drv->name, bflsc->device_id, xlink, tmp);
+			applog(LOG_INFO, "%s%i:%s invalid nonce (%s) will try to process anyway",
+			       bflsc->drv->name, bflsc->device_id, xlink, tmp);
 			free(tmp);
 		}
 
@@ -1543,14 +1813,16 @@ static void bflsc_thread_enable(struct thr_info *thr)
 	bflsc_initialise(bflsc);
 }
 
-static bool bflsc_send_work(struct cgpu_info *bflsc, int dev, struct work *work)
+static bool bflsc_send_work(struct cgpu_info *bflsc, int dev, struct work *work,
+			    bool mandatory)
 {
 	struct bflsc_info *sc_info = (struct bflsc_info *)(bflsc->device_data);
 	struct FullNonceRangeJob data;
 	char buf[BFLSC_BUFSIZ+1];
 	int err, amount;
-	int len;
-	int try;
+	int len, try;
+	int stage;
+	bool sent;
 
 	// Device is gone
 	if (bflsc->usbinfo.nodev)
@@ -1566,10 +1838,66 @@ static bool bflsc_send_work(struct cgpu_info *bflsc, int dev, struct work *work)
 	memcpy(data.blockData, work->data + MERKLE_OFFSET, MERKLE_BYTES);
 	data.endOfBlock = BFLSC_EOB;
 
-	try = 0;
+	len = sizeof(struct FullNonceRangeJob);
 
-	mutex_lock(&(bflsc->device_mutex));
+	/* On faster devices we have a lot of lock contention so only
+	 * mandatorily grab the lock and send work if the queue is empty since
+	 * we have a submit queue. */
+	if (mandatory)
+		mutex_lock(&(bflsc->device_mutex));
+	else {
+		if (mutex_trylock(&bflsc->device_mutex))
+			return false;
+	}
+
+	try = 0;
 re_send:
+	err = send_recv_ds(bflsc, dev, &stage, &sent, &amount,
+				BFLSC_QJOB, BFLSC_QJOB_LEN, C_REQUESTQUEJOB, C_REQUESTQUEJOBSTATUS,
+				(char *)&data, len, C_QUEJOB, C_QUEJOBSTATUS,
+				buf, sizeof(buf)-1);
+	mutex_unlock(&(bflsc->device_mutex));
+
+	switch (stage) {
+		case 1:
+			if (!sent) {
+				bflsc_applog(bflsc, dev, C_REQUESTQUEJOB, amount, err);
+				return false;
+			} else {
+				// TODO: handle other errors ...
+
+				// Try twice
+				if (try++ < 1 && amount > 1 &&
+					strncasecmp(buf, BFLSC_TIMEOUT, BFLSC_TIMEOUT_LEN) == 0)
+						goto re_send;
+
+				bflsc_applog(bflsc, dev, C_REQUESTQUEJOBSTATUS, amount, err);
+				return false;
+			}
+			break;
+		case 2:
+			if (!sent) {
+				bflsc_applog(bflsc, dev, C_QUEJOB, amount, err);
+				return false;
+			} else {
+				if (!isokerr(err, buf, amount)) {
+					// TODO: check for QUEUE FULL and set work_queued to sc_info->que_size
+					//  and report a code bug LOG_ERR - coz it should never happen
+					// TODO: handle other errors ...
+
+					// Try twice
+					if (try++ < 1 && amount > 1 &&
+						strncasecmp(buf, BFLSC_TIMEOUT, BFLSC_TIMEOUT_LEN) == 0)
+							goto re_send;
+
+					bflsc_applog(bflsc, dev, C_QUEJOBSTATUS, amount, err);
+					return false;
+				}
+			}
+			break;
+	}
+
+/*
 	err = write_to_dev(bflsc, dev, BFLSC_QJOB, BFLSC_QJOB_LEN, &amount, C_REQUESTQUEJOB);
 	if (err < 0 || amount != BFLSC_QJOB_LEN) {
 		mutex_unlock(&(bflsc->device_mutex));
@@ -1593,7 +1921,7 @@ re_send:
 	}
 
 	if (!getokerr(bflsc, C_QUEJOBSTATUS, &err, &amount, buf, sizeof(buf))) {
-		// TODO: check for QUEUE FULL and set work_queued to BFLSC_QUE_SIZE
+		// TODO: check for QUEUE FULL and set work_queued to sc_info->que_size
 		//  and report a code bug LOG_ERR - coz it should never happen
 
 		// Try twice
@@ -1607,6 +1935,7 @@ re_send:
 	}
 
 	mutex_unlock(&(bflsc->device_mutex));
+*/
 
 	wr_lock(&(sc_info->stat_lock));
 	sc_info->sc_devs[dev].work_queued++;
@@ -1629,6 +1958,8 @@ static bool bflsc_queue_full(struct cgpu_info *bflsc)
 	// if something is wrong with a device try the next one available
 	// TODO: try them all? Add an unavailable flag to sc_devs[i] init to 0 here first
 	while (++tries < 3) {
+		bool mandatory = false;
+
 		// Device is gone - shouldn't normally get here
 		if (bflsc->usbinfo.nodev) {
 			ret = true;
@@ -1648,7 +1979,7 @@ static bool bflsc_queue_full(struct cgpu_info *bflsc)
 		}
 
 		if (dev == -1) {
-			que = BFLSC_QUE_SIZE * 10; // 10x is certainly above the MAX it could be
+			que = sc_info->que_size * 10; // 10x is certainly above the MAX it could be
 			// The first device with the smallest amount queued
 			for (i = 0; i < sc_info->sc_count; i++) {
 				if (i != tried && sc_info->sc_devs[i].work_queued < que &&
@@ -1657,8 +1988,10 @@ static bool bflsc_queue_full(struct cgpu_info *bflsc)
 					que = sc_info->sc_devs[i].work_queued;
 				}
 			}
-			if (que > BFLSC_QUE_FULL_ENOUGH)
+			if (que > sc_info->que_full_enough)
 				dev = -1;
+			else if (que < sc_info->que_low)
+				mandatory = true;
 		}
 		rd_unlock(&(sc_info->stat_lock));
 
@@ -1672,7 +2005,7 @@ static bool bflsc_queue_full(struct cgpu_info *bflsc)
 			work = get_queued(bflsc);
 		if (unlikely(!work))
 			break;
-		if (bflsc_send_work(bflsc, dev, work)) {
+		if (bflsc_send_work(bflsc, dev, work, mandatory)) {
 			work = NULL;
 			break;
 		} else
@@ -1740,10 +2073,10 @@ static int64_t bflsc_scanwork(struct thr_info *thr)
 	waited = restart_wait(sc_info->scan_sleep_time);
 	if (waited == ETIMEDOUT) {
 		unsigned int old_sleep_time, new_sleep_time = 0;
-		int min_queued = BFLSC_QUE_SIZE;
+		int min_queued = sc_info->que_size;
 		/* Only adjust the scan_sleep_time if we did not receive a
 		 * restart message while waiting. Try to adjust sleep time
-		 * so we drop to BFLSC_QUE_WATERMARK before getting more work.
+		 * so we drop to sc_info->que_watermark before getting more work.
 		 */
 
 		rd_lock(&sc_info->stat_lock);
@@ -1756,9 +2089,9 @@ static int64_t bflsc_scanwork(struct thr_info *thr)
 		new_sleep_time = old_sleep_time;
 
 		/* Increase slowly but decrease quickly */
-		if (min_queued > BFLSC_QUE_WATERMARK && old_sleep_time < BFLSC_MAX_SLEEP)
+		if (min_queued > sc_info->que_full_enough && old_sleep_time < BFLSC_MAX_SLEEP)
 			new_sleep_time = old_sleep_time * 21 / 20;
-		else if (min_queued < BFLSC_QUE_WATERMARK)
+		else if (min_queued < sc_info->que_watermark)
 			new_sleep_time = old_sleep_time * 2 / 3;
 
 		/* Do not sleep more than BFLSC_MAX_SLEEP so we can always
@@ -1792,26 +2125,38 @@ static int64_t bflsc_scanwork(struct thr_info *thr)
 	return ret;
 }
 
+#define BFLSC_OVER_TEMP 60
+
+/* Set the fanspeed to auto for any valid value <= BFLSC_OVER_TEMP,
+ * or max for any value > BFLSC_OVER_TEMP or if we don't know the temperature. */
 static void bflsc_set_fanspeed(struct cgpu_info *bflsc)
 {
 	struct bflsc_info *sc_info = (struct bflsc_info *)bflsc->device_data;
-	int amount, err;
+	char buf[BFLSC_BUFSIZ+1];
+	char data[16+1];
+	int amount;
+	bool sent;
 
-	if ((bflsc->temp <= 60 && sc_info->fanauto) ||
-	    (bflsc->temp > 60 && !sc_info->fanauto))
+	if ((bflsc->temp <= BFLSC_OVER_TEMP && bflsc->temp > 0 && sc_info->fanauto) ||
+	    ((bflsc->temp > BFLSC_OVER_TEMP || !bflsc->temp) && !sc_info->fanauto))
 		return;
 
-	mutex_lock(&bflsc->device_mutex);
-	if (bflsc->temp > 60) {
-		write_to_dev(bflsc, 0, BFLSC_FAN4, BFLSC_FAN4_LEN, &amount,
-			     C_SETFAN);
+	if (bflsc->temp > BFLSC_OVER_TEMP || !bflsc->temp) {
+		strcpy(data, BFLSC_FAN4);
 		sc_info->fanauto = false;
 	} else {
-		write_to_dev(bflsc, 0, BFLSC_FANAUTO, BFLSC_FANOUT_LEN,
-			     &amount, C_SETFAN);
+		strcpy(data, BFLSC_FANAUTO);
 		sc_info->fanauto = true;
 	}
-	getok(bflsc, C_FANREPLY, &err, &amount);
+
+	applog(LOG_DEBUG, "%s%i: temp=%.0f over=%d set fan to %s",
+				bflsc->drv->name, bflsc->device_id, bflsc->temp,
+				BFLSC_OVER_TEMP, data);
+
+	mutex_lock(&bflsc->device_mutex);
+	send_recv_ss(bflsc, 0, &sent, &amount,
+				data, strlen(data), C_SETFAN,
+				buf, sizeof(buf)-1, C_FANREPLY, READ_NL);
 	mutex_unlock(&bflsc->device_mutex);
 }
 
@@ -1870,6 +2215,7 @@ static struct api_data *bflsc_api_stats(struct cgpu_info *bflsc)
 {
 	struct bflsc_info *sc_info = (struct bflsc_info *)(bflsc->device_data);
 	struct api_data *root = NULL;
+	int i;
 
 //if no x-link ... etc
 	rd_lock(&(sc_info->stat_lock));
@@ -1882,7 +2228,25 @@ static struct api_data *bflsc_api_stats(struct cgpu_info *bflsc)
 	root = api_add_temp(root, "Temp2 Max", &(sc_info->sc_devs[0].temp2_max), true);
 	root = api_add_time(root, "Temp1 Max Time", &(sc_info->sc_devs[0].temp1_max_time), true);
 	root = api_add_time(root, "Temp2 Max Time", &(sc_info->sc_devs[0].temp2_max_time), true);
+	root = api_add_int(root, "Work Queued", &(sc_info->sc_devs[0].work_queued), true);
+	root = api_add_int(root, "Work Complete", &(sc_info->sc_devs[0].work_complete), true);
+	root = api_add_bool(root, "Overheat", &(sc_info->sc_devs[0].overheat), true);
+	root = api_add_uint64(root, "Flush ID", &(sc_info->sc_devs[0].flush_id), true);
+	root = api_add_uint64(root, "Result ID", &(sc_info->sc_devs[0].result_id), true);
+	root = api_add_bool(root, "Flushed", &(sc_info->sc_devs[0].flushed), true);
+	root = api_add_uint(root, "Scan Sleep", &(sc_info->scan_sleep_time), true);
+	root = api_add_uint(root, "Results Sleep", &(sc_info->results_sleep_time), true);
+	root = api_add_uint(root, "Work ms", &(sc_info->default_ms_work), true);
 	rd_unlock(&(sc_info->stat_lock));
+
+	i = (int)(sc_info->driver_version);
+	root = api_add_int(root, "Driver", &i, true);
+	root = api_add_string(root, "Firmware", sc_info->sc_devs[0].firmware, false);
+	root = api_add_string(root, "Chips", sc_info->sc_devs[0].chips, false);
+	root = api_add_int(root, "Que Size", &(sc_info->que_size), false);
+	root = api_add_int(root, "Que Full", &(sc_info->que_full_enough), false);
+	root = api_add_int(root, "Que Watermark", &(sc_info->que_watermark), false);
+	root = api_add_int(root, "Que Low", &(sc_info->que_low), false);
 	root = api_add_escape(root, "GetInfo", sc_info->sc_devs[0].getinfo, false);
 
 /*
