@@ -61,7 +61,7 @@
 #define AVALON_TIMEOUT_MS 999
 #define ICARUS_TIMEOUT_MS 999
 #else
-#define BFLSC_TIMEOUT_MS 200
+#define BFLSC_TIMEOUT_MS 300
 #define BITFORCE_TIMEOUT_MS 200
 #define MODMINER_TIMEOUT_MS 100
 #define AVALON_TIMEOUT_MS 200
@@ -147,7 +147,7 @@ static struct usb_find_devices find_dev[] = {
 		.ident = IDENT_BAS,
 		.idVendor = IDVENDOR_FTDI,
 		.idProduct = 0x6014,
-		.iManufacturer = "Butterfly Labs",
+		//.iManufacturer = "Butterfly Labs",
 		.iProduct = "BitFORCE SHA256 SC",
 		.kernel = 0,
 		.config = 1,
@@ -305,7 +305,7 @@ static struct usb_find_devices find_dev[] = {
 		.epcount = 0,
 		.eps = NULL },
 #endif
-	{ DRV_LAST, NULL, 0, 0, 0, NULL, NULL, 0, 0, 0, 0, 0, 0, NULL }
+	{ DRV_LAST, NULL, 0, 0, 0, NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL }
 };
 
 #ifdef USE_BFLSC
@@ -1046,7 +1046,6 @@ void usb_applog(struct cgpu_info *cgpu, enum usb_cmds cmd, char *msg, int amount
                         err, amount);
 }
 
-#ifdef WIN32
 static void in_use_store_ress(uint8_t bus_number, uint8_t device_address, void *resource1, void *resource2)
 {
 	struct usb_in_use_list *in_use_tmp;
@@ -1118,7 +1117,6 @@ static void in_use_get_ress(uint8_t bus_number, uint8_t device_address, void **r
 		applog(LOG_ERR, "FAIL: USB get_lock empty (%d:%d)",
 				(int)bus_number, (int)device_address);
 }
-#endif
 
 static bool __is_in_use(uint8_t bus_number, uint8_t device_address)
 {
@@ -1465,13 +1463,13 @@ struct cgpu_info *usb_free_cgpu_devlock(struct cgpu_info *cgpu, bool free_devloc
  *  the thread Cancelability unrestored
  */
 #define DEVLOCK(cgpu, _pth_state) do { \
-			pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &_pth_state); \
+			pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &_pth_state); \
 			wr_lock(cgpu->usbinfo.devlock); \
 			} while (0)
 
 #define DEVUNLOCK(cgpu, _pth_state) do { \
 			wr_unlock(cgpu->usbinfo.devlock); \
-			pthread_setcanceltype(_pth_state, NULL); \
+			pthread_setcancelstate(_pth_state, NULL); \
 			} while (0)
 
 static int _usb_init(struct cgpu_info *cgpu, struct libusb_device *dev, struct usb_find_devices *found)
@@ -1649,6 +1647,7 @@ static int _usb_init(struct cgpu_info *cgpu, struct libusb_device *dev, struct u
 					&&  epdesc->wMaxPacketSize >= found->eps[k].size
 					&&  epdesc->bEndpointAddress == found->eps[k].ep) {
 						found->eps[k].found = true;
+						found->wMaxPacketSize = epdesc->wMaxPacketSize;
 						break;
 					}
 				}
@@ -2181,15 +2180,31 @@ static int
 usb_bulk_transfer(struct libusb_device_handle *dev_handle,
 		  unsigned char endpoint, unsigned char *data, int length,
 		  int *transferred, unsigned int timeout,
-		  struct cgpu_info *cgpu)
+		  struct cgpu_info *cgpu, enum usb_cmds cmd)
 {
-	int err, tries = 0;
+	uint16_t MaxPacketSize;
+	int err, errn, tries = 0;
+
+	/* Limit length of transfer to the largest this descriptor supports
+	 * and leave the higher level functions to transfer more if needed. */
+	if (cgpu->usbdev->PrefPacketSize)
+		MaxPacketSize = cgpu->usbdev->PrefPacketSize;
+	else
+		MaxPacketSize = cgpu->usbdev->found->wMaxPacketSize;
+	if (length > MaxPacketSize)
+		length = MaxPacketSize;
 
 	cg_rlock(&cgusb_fd_lock);
 	err = libusb_bulk_transfer(dev_handle, endpoint, data, length,
 				   transferred, timeout);
+	errn = errno;
 	cg_runlock(&cgusb_fd_lock);
 
+	if (err < 0)
+		applog(LOG_DEBUG, "%s%i: %s (amt=%d err=%d ern=%d)",
+				cgpu->drv->name, cgpu->device_id,
+				usb_cmdname(cmd), *transferred, err, errn);
+
 	if (err == LIBUSB_ERROR_PIPE) {
 		cgpu->usbinfo.last_pipe = time(NULL);
 		cgpu->usbinfo.pipe_count++;
@@ -2206,8 +2221,14 @@ usb_bulk_transfer(struct libusb_device_handle *dev_handle,
 			cg_rlock(&cgusb_fd_lock);
 			err = libusb_bulk_transfer(dev_handle, endpoint, data,
 						   length, transferred, timeout);
+			errn = errno;
 			cg_runlock(&cgusb_fd_lock);
 
+			if (err < 0)
+				applog(LOG_DEBUG, "%s%i: %s (amt=%d err=%d ern=%d)",
+						cgpu->drv->name, cgpu->device_id,
+						usb_cmdname(cmd), *transferred, err, errn);
+
 			if (err)
 				cgpu->usbinfo.retry_err_count++;
 		} while (err == LIBUSB_ERROR_PIPE && tries++ < USB_RETRY_MAX);
@@ -2221,7 +2242,7 @@ usb_bulk_transfer(struct libusb_device_handle *dev_handle,
 	return err;
 }
 
-int _usb_read(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *processed, unsigned int timeout, const char *end, __maybe_unused enum usb_cmds cmd, bool readonce)
+int _usb_read(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *processed, unsigned int timeout, const char *end, enum usb_cmds cmd, bool readonce)
 {
 	struct cg_usb_device *usbdev;
 	bool ftdi;
@@ -2295,7 +2316,7 @@ int _usb_read(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *pro
 			err = usb_bulk_transfer(usbdev->handle,
 						usbdev->found->eps[ep].ep,
 						ptr, usbbufread, &got, timeout,
-						cgpu);
+						cgpu, cmd);
 			cgtime(&tv_finish);
 			USB_STATS(cgpu, &tv_start, &tv_finish, err,
 					MODE_BULK_READ, cmd, first ? SEQ0 : SEQ1);
@@ -2385,7 +2406,8 @@ int _usb_read(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *pro
 		STATS_TIMEVAL(&tv_start);
 		err = usb_bulk_transfer(usbdev->handle,
 					usbdev->found->eps[ep].ep, ptr,
-					usbbufread, &got, timeout, cgpu);
+					usbbufread, &got, timeout,
+					cgpu, cmd);
 		cgtime(&tv_finish);
 		USB_STATS(cgpu, &tv_start, &tv_finish, err,
 				MODE_BULK_READ, cmd, first ? SEQ0 : SEQ1);
@@ -2472,7 +2494,7 @@ out_unlock:
 	return err;
 }
 
-int _usb_write(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *processed, unsigned int timeout, __maybe_unused enum usb_cmds cmd)
+int _usb_write(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *processed, unsigned int timeout, enum usb_cmds cmd)
 {
 	struct cg_usb_device *usbdev;
 #if DO_USB_STATS
@@ -2513,7 +2535,7 @@ int _usb_write(struct cgpu_info *cgpu, int ep, char *buf, size_t bufsiz, int *pr
 		err = usb_bulk_transfer(usbdev->handle,
 					usbdev->found->eps[ep].ep,
 					(unsigned char *)buf, bufsiz, &sent,
-					timeout, cgpu);
+					timeout, cgpu, cmd);
 		cgtime(&tv_finish);
 		USB_STATS(cgpu, &tv_start, &tv_finish, err,
 				MODE_BULK_WRITE, cmd, first ? SEQ0 : SEQ1);
@@ -2923,7 +2945,7 @@ void usb_initialise()
 				if (bus <= 0)
 					quit(1, "Invalid --usb bus:dev - bus must be > 0");
 
-				if (!colon == '*')
+				if (*colon == '*')
 					dev = -1;
 				else {
 					dev = atoi(colon);
@@ -3176,8 +3198,9 @@ fail:
 	struct semid_ds seminfo;
 	union semun opt;
 	char name[64];
-	key_t key;
-	int fd, sem, count;
+	key_t *key;
+	int *sem;
+	int fd, count;
 
 	if (is_in_use_bd(bus_number, device_address))
 		return false;
@@ -3188,25 +3211,34 @@ fail:
 		applog(LOG_ERR,
 			"SEM: %s USB open failed '%s' err (%d) %s",
 			dname, name, errno, strerror(errno));
-		return false;
+		goto _out;
 	}
 	close(fd);
-	key = ftok(name, 'K');
-	sem = semget(key, 1, IPC_CREAT | IPC_EXCL | 438);
-	if (sem < 0) {
+
+	key = malloc(sizeof(*key));
+	if (unlikely(!key))
+		quit(1, "SEM: Failed to malloc key");
+
+	sem = malloc(sizeof(*sem));
+	if (unlikely(!sem))
+		quit(1, "SEM: Failed to malloc sem");
+
+	*key = ftok(name, 'K');
+	*sem = semget(*key, 1, IPC_CREAT | IPC_EXCL | 438);
+	if (*sem < 0) {
 		if (errno != EEXIST) {
 			applog(LOG_ERR,
 				"SEM: %s USB failed to get '%s' err (%d) %s",
 				dname, name, errno, strerror(errno));
-			return false;
+			goto free_out;
 		}
 
-		sem = semget(key, 1, 0);
-		if (sem < 0) {
+		*sem = semget(*key, 1, 0);
+		if (*sem < 0) {
 			applog(LOG_ERR,
 				"SEM: %s USB failed to access '%s' err (%d) %s",
 				dname, name, errno, strerror(errno));
-			return false;
+			goto free_out;
 		}
 
 		opt.buf = &seminfo;
@@ -3216,14 +3248,14 @@ fail:
 			if (count > 99) {
 				applog(LOG_ERR,
 					"SEM: %s USB timeout waiting for (%d) '%s'",
-					dname, sem, name);
-				return false;
+					dname, *sem, name);
+				goto free_out;
 			}
-			if (semctl(sem, 0, IPC_STAT, opt) == -1) {
+			if (semctl(*sem, 0, IPC_STAT, opt) == -1) {
 				applog(LOG_ERR,
 					"SEM: %s USB failed to wait for (%d) '%s' count %d err (%d) %s",
-					dname, sem, name, count, errno, strerror(errno));
-				return false;
+					dname, *sem, name, count, errno, strerror(errno));
+				goto free_out;
 			}
 			if (opt.buf->sem_otime != 0)
 				break;
@@ -3236,22 +3268,29 @@ fail:
 		{ 0, 1, IPC_NOWAIT | SEM_UNDO }
 	};
 
-	if (semop(sem, sops, 2)) {
+	if (semop(*sem, sops, 2)) {
 		if (errno == EAGAIN) {
 			if (!hotplug_mode)
 				applog(LOG_WARNING,
 					"SEM: %s USB failed to get (%d) '%s' - device in use",
-					dname, sem, name);
+					dname, *sem, name);
 		} else {
 			applog(LOG_DEBUG,
 				"SEM: %s USB failed to get (%d) '%s' err (%d) %s",
-				dname, sem, name, errno, strerror(errno));
+				dname, *sem, name, errno, strerror(errno));
 		}
-		return false;
+		goto free_out;
 	}
 
 	add_in_use(bus_number, device_address);
+	in_use_store_ress(bus_number, device_address, (void *)key, (void *)sem);
 	return true;
+
+free_out:
+	free(sem);
+	free(key);
+_out:
+	return false;
 #endif
 }
 
@@ -3288,27 +3327,15 @@ fila:
 	return;
 #else
 	char name[64];
-	key_t key;
-	int fd, sem;
+	key_t *key = NULL;
+	int *sem = NULL;
 
 	sprintf(name, "/tmp/cgminer-usb-%d-%d", (int)bus_number, (int)device_address);
-	fd = open(name, O_CREAT|O_RDONLY, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
-	if (fd == -1) {
-		applog(LOG_ERR,
-			"SEM: %s USB open failed '%s' for release err (%d) %s",
-			dname, name, errno, strerror(errno));
-		return;
-	}
-	close(fd);
-	key = ftok(name, 'K');
 
-	sem = semget(key, 1, 0);
-	if (sem < 0) {
-		applog(LOG_ERR,
-			"SEM: %s USB failed to get '%s' for release err (%d) %s",
-			dname, name, errno, strerror(errno));
-		return;
-	}
+	in_use_get_ress(bus_number, device_address, (void **)(&key), (void **)(&sem));
+
+	if (!key || !sem)
+		goto fila;
 
 	struct sembuf sops[] = {
 		{ 0, -1, SEM_UNDO }
@@ -3318,12 +3345,22 @@ fila:
 	// exceeding this timeout means it would probably never succeed anyway
 	struct timespec timeout = { 0, 10000000 };
 
-	if (semtimedop(sem, sops, 1, &timeout)) {
+	if (semtimedop(*sem, sops, 1, &timeout)) {
 		applog(LOG_ERR,
 			"SEM: %s USB failed to release '%s' err (%d) %s",
 			dname, name, errno, strerror(errno));
 	}
 
+	if (semctl(*sem, 0, IPC_RMID)) {
+		applog(LOG_WARNING,
+			"SEM: %s USB failed to remove SEM '%s' err (%d) %s",
+			dname, name, errno, strerror(errno));
+	}
+
+fila:
+
+	free(sem);
+	free(key);
 	remove_in_use(bus_number, device_address);
 	return;
 #endif
